from contextlib import asynccontextmanager
from datetime import datetime

from fastapi import FastAPI
from py_tools.connections.http import AsyncHttpClient
from py_tools.logging import logger

from .config.site import SiteConfig
from .middlewares.base import register_middlewares
from .router.knowledge_nodes_api import knowledge_nodes_api_router
from .utils import log_util


@asynccontextmanager
async def lifespan(app: FastAPI):
    await startup()
    yield
    await shutdown()


app = FastAPI(
    description="知识图谱开放平台",
    lifespan=lifespan,
    middleware=register_middlewares(),  # 注册web中间件
)

@app.get("/health")
async def health_check():
    """健康检查接口"""
    return {
        "status": "ok",
        "timestamp": datetime.utcnow().isoformat(),
        "service": "knowledge-graph"
    }


async def init_setup():
    """初始化项目配置"""

    log_util.setup_logger()
    
async def startup():
    """项目启动时准备环境"""

    await init_setup()

    # 加载路由
    app.include_router(knowledge_nodes_api_router)

    logger.info("fastapi startup success")


async def shutdown():
    await AsyncHttpClient.close()
    logger.error("app shutdown")
