# @Desc: { 模块描述 }
import json
import time

from fastapi import Request, status
from fastapi.middleware import Middleware
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import Response
from py_tools.logging import logger
from starlette.middleware.base import BaseHTTPMiddleware, RequestResponseEndpoint

from ..config.site import SiteConfig
from ..service.dict_system_service import DictSystemService
from ..settings import auth_setting
from ..utils.trace_util import TraceUtil
from typing import Optional
from cachetools import TTLCache

class LoggingMiddleware(BaseHTTPMiddleware):
    """
    日志中间件
    记录请求参数信息、计算响应时间
    """

    async def set_body(self, request: Request):
        receive_ = await request._receive()

        async def receive():
            return receive_

        request._receive = receive

    async def dispatch(self, request: Request, call_next) -> Response:
        start_time = time.perf_counter()

        # 打印请求信息
        request_id = TraceUtil.get_req_id()
        logger.info(f"--> {request.method} {request.url.path} {request.client.host}")
        if request.query_params:
            logger.info(f"--> Query Params: {request.query_params}")

        if "application/json" in request.headers.get("Content-Type", ""):
            await self.set_body(request)
            try:
                # starlette 中间件中不能读取请求数据，否则会进入循环等待 需要特殊处理或者换APIRoute实现
                body = await request.json()
                logger.info(f"--> Request Body: {body}")
            except Exception as e:
                logger.warning(f"Failed to parse JSON body: {e}")

        # 执行请求获取响应
        response = await call_next(request)

        # 计算响应时间
        process_time = time.perf_counter() - start_time
        response.headers["X-Response-Time"] = f"{process_time:.2f}s"

        # 记录响应基本信息
        logger.info(f"<-- Response: [Status: {response.status_code}, Type: {response.__class__.__name__}, Headers: {dict(response.headers)}]")

        # 尝试记录响应体
        try:
            from starlette.responses import StreamingResponse, JSONResponse, Response
            import json

            # 针对不同类型的响应采用不同的解析方式
            if isinstance(response, StreamingResponse):
                logger.info("<-- Streaming response body (skipped logging)")
                return response

            # 获取响应体内容
            body = None
            if isinstance(response, JSONResponse):
                body = response.body
            elif hasattr(response, 'body'):
                try:
                    body = await response.body()
                except Exception as e:
                    logger.warning(f"<-- Failed to get response body: {e.__class__.__name__}: {str(e)}")
                    return response

            # 解析并记录响应体
            if body:
                try:
                    if isinstance(body, bytes):
                        body = body.decode('utf-8')
                    if isinstance(body, str):
                        # 尝试解析为JSON
                        try:
                            body_json = json.loads(body)
                            logger.info(f"<-- Response Body (JSON): {body_json}")
                        except json.JSONDecodeError:
                            # 如果不是JSON格式，直接记录字符串内容
                            logger.info(f"<-- Response Body (Text): {body}")
                except UnicodeDecodeError as e:
                    logger.warning(f"<-- Failed to decode response body: {str(e)}")
                except Exception as e:
                    logger.warning(f"<-- Failed to process response body: {e.__class__.__name__}: {str(e)}")
        except Exception as e:
            logger.warning(f"<-- Failed to handle response body: {e.__class__.__name__}: {str(e)}")


        logger.info(f"<-- {response.status_code} {request.url.path} (took: {process_time:.2f}s)")

        return response


class TraceReqMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next: RequestResponseEndpoint) -> Response:
        # 设置请求id
        request_id = TraceUtil.set_req_id(request.headers.get("X-Request-ID"))
        response = await call_next(request)
        response.headers["X-Request-ID"] = f"{request_id}"  # 记录同一个请求的唯一id
        return response


class AuthMiddleware(BaseHTTPMiddleware):
    """鉴权中间件"""

    @staticmethod
    def set_auth_err_resp(err_msg: str):
        return Response(
            content=err_msg,
            status_code=status.HTTP_401_UNAUTHORIZED
        )

    async def verify_token(self, authorization: str) -> Optional[dict]:
        """
        验证 token 有效性
        返回：验证成功返回用户信息字典，失败返回 None
        """
        if not authorization or not authorization.startswith("Bearer "):
            return None

        token = authorization[7:]
        # 从环境变量获取预设的token值进行比对
        admin_token = auth_setting.admin_token
        user_token = auth_setting.user_token

        if token == admin_token:
            return {"id": 1, "username": "admin", "role": "admin"}
        elif token == user_token:
            return {"id": 2, "username": "user", "role": "user"}
        return None

    def should_intercept(self, path: str) -> bool:
        """
        判断是否需要拦截当前路径
        """
        if path in auth_setting.auth_whitelist_urls:
            return False

        for pattern in auth_setting.auth_blacklist_urls:
            # 处理通配符匹配
            if pattern.endswith("/*"):
                if path.startswith(pattern[:-1]):
                    return True
            # 精确匹配
            elif path == pattern:
                return True
        return False

    _cache = TTLCache(1, 360)
    async def dispatch(self, request: Request, call_next: RequestResponseEndpoint) -> Response:
        # 初始化请求上下文
        request.state.context = {
            "request_id": TraceUtil.get_req_id(),
            "client_ip": request.client.host
        }

        path = request.url.path

        # if not self.should_intercept(path):
        #     return await call_next(request)
        # 
        # # 权限校验
        # auth_header = request.headers.get("Authorization")
        # if not auth_header:
        #     return self.set_auth_err_resp("Missing Authorization header")
        # 
        # user_info = await self.verify_token(auth_header)
        # if not user_info:
        #     return self.set_auth_err_resp("Invalid token")

        # 初始化操作：将用户信息添加到请求状态中
        #request.state.user = user_info
        #cache["license_info"]是否存在license信息，如果不存在则验证证书
        # if not 'license_info' in self._cache or not self._cache["license_info"]:
        #     lisence_detail = license_handle()
        #     #判断license_info是否是个json，且存在key为expiration_time的字段，如果存在则判断是否过期
        #     if lisence_detail and "expiration_time" in lisence_detail:
        #         license_info = json.loads(lisence_detail)
        #         expiration_time = license_info.get("expiration_time")
        #         if expiration_time and expiration_time < time.time():
        #             return self.set_auth_err_resp("License expired")
        #         else:
        #             self._cache["license_info"] = license_info
        #     else:
        #         return self.set_auth_err_resp(lisence_detail)
        #
        #     if hasattr(self, '_invok_info'):
        #         current_count = self._invok_info.get('api_invoke_count', 0)
        #           # 从字典服务获取调用次数的枚举值
        #         dict_service = DictSystemService(self.db)
        #         try:
        #             call_count_dict = dict_service.get_dicts_by_name('api_invoke_count')
        #             old_increment_value = int(call_count_dict.dict_value)
        #
        #             # 更新调用次数
        #             updated_count = current_count + old_increment_value
        #
        #             # 更新数据库中的调用次数
        #             dict_service.update_dict(call_count_dict['id'], {'dict_value': str(updated_count)})
        #             self._invok_info['api_invoke_count'] = 0
        #
        #             api_invoke_max_count = license_info.get("content")[0].get("api_invoke_max_count")
        #             if updated_count >= api_invoke_max_count:
        #                 return self.set_auth_err_resp("调用次数已达上限!")
        #         except Exception as e:
        #             logger.error(f"更新调用次数失败: {str(e)}")
                
        # 继续处理请求
        response = await call_next(request)

        # 统计调用次数
        # if not hasattr(self, '_invok_info'):
        #     self._invok_info = TTLCache(maxsize=20)
        #
        # current_count = self._invok_info.get('api_invoke_count', 0)
        # self._invok_info['api_invoke_count'] = current_count + 1
        # 可以在返回前添加统一响应处理（如添加头信息）
        # response.headers["request-id"] = request.state.context["request_id"]

        return response

def license_handle():
    """验证证书"""
    
    license_dir = SiteConfig().get_config("license")
        
    try:
        if not license_dir:
            return "license目录未配置"
            
        with open(f"{license_dir}/public.key", "rb") as f:
            public_key_pem = f.read()
        with open(f"{license_dir}/license_issued.lic", "rb") as f:
            data = json.loads(f.read())
            license_json = json.dumps(data, sort_keys=True).encode()
        with open(f"{license_dir}/license_issued.key", "rb") as f:
            signature = f.read()
            
        from ..utils.license import validate_license
        if not validate_license(public_key_pem, license_json, signature):
            return "license验证失败"
            
        # 验证成功，将license_json存入缓存
        return license_json
        
    except Exception as e:
        # 验证失败，将错误信息存入缓存
        return str(e)

def register_middlewares():
    """注册中间件（逆序执行）"""
    return [
        Middleware(TraceReqMiddleware),  # 最先设置request_id
        Middleware(LoggingMiddleware),
        Middleware(
            CORSMiddleware,
            allow_origins=["*"],
            allow_credentials=True,
            allow_methods=["*"],
            allow_headers=["*"],
        ),
        Middleware(AuthMiddleware),
    ]
