from fastapi import APIRouter, HTTPException, Depends
from pydantic import BaseModel, Field, validator
from typing import List, Optional
from ..service.trunks_service import TrunksService
from ..utils.sentence_util import SentenceUtil

from ..model.response import StandardResponse

import os

import logging

from ..db.session import get_db
from sqlalchemy.orm import Session


from cachetools import TTLCache

# 使用TextSimilarityFinder进行文本相似度匹配
from ..utils.text_similarity import TextSimilarityFinder

logger = logging.getLogger(__name__)
router = APIRouter(tags=["Text Search"])
DISTANCE_THRESHOLD = 0.73
# 创建全局缓存实例
#cache = TTLCache(maxsize=1000, ttl=3600)

class FindSimilarTexts(BaseModel):
    keywords:Optional[List[str]] = None
    search_text: str

@router.post("/knowledge/text/find_similar_texts", response_model=StandardResponse)
async def find_similar_texts(request: FindSimilarTexts, db: Session = Depends(get_db)):
    trunks_service = TrunksService()
    search_text = request.search_text
    if request.keywords:
        search_text = f"{request.keywords}:{search_text}"
    # 使用向量搜索获取相似内容
    search_results = trunks_service.search_by_vector(
        text=search_text,
        limit=500,
        type='trunk',
        distance=0.7
    )

    # 准备语料库数据
    trunk_texts = []
    trunk_ids = []

    # 创建一个字典来存储trunk的详细信息
    trunk_details = {}
    if len(search_results) == 0:
        return StandardResponse(success=True)
    for trunk in search_results:
        trunk_texts.append(trunk.get('content'))
        trunk_ids.append(trunk.get('id'))
        # 缓存trunk的详细信息
        trunk_details[trunk.get('id')] = {
            'id': trunk.get('id'),
            'content': trunk.get('content'),
            'file_path': trunk.get('file_path'),
            'title': trunk.get('title'),
            'referrence': trunk.get('referrence'),
            'page_no': trunk.get('page_no')
        }

    # 初始化TextSimilarityFinder并加载语料库
    similarity_finder = TextSimilarityFinder(method='tfidf', use_jieba=True)
    similarity_finder.load_corpus(trunk_texts, trunk_ids)

    similar_results = similarity_finder.find_most_similar(search_text, top_n=1)
    prop_result = {}
    # 处理搜索结果
    if similar_results and similar_results[0]['similarity'] >= 0.3:  # 设置相似度阈值
        # 获取最相似的文本对应的trunk_id
        trunk_id = similar_results[0]['path']

        # 从缓存中获取trunk详细信息
        trunk_info = trunk_details.get(trunk_id)

        if trunk_info:
            search_result = {
                **trunk_info,
                'distance': similar_results[0]['similarity']  # 转换相似度为距离
            }

            reference, _ = _process_search_result(search_result, 1)
            prop_result["references"] = [reference]
            prop_result["answer"] = [{
                 "sentence": request.search_text,
                 "index": "1"
            }]
    else:
        # 如果整体搜索没有找到匹配结果，则进行句子拆分搜索
        sentences = SentenceUtil.split_text(request.search_text, 10)
        result_sentences, references = _process_sentence_search_keywords(
            sentences, trunks_service,keywords=request.keywords
        )
        if references:
            prop_result["references"] = references
        if result_sentences:
            prop_result["answer"] = result_sentences
    return StandardResponse(success=True,data=prop_result)

def _process_search_result(search_result: dict, reference_index: int) -> tuple[dict, str]:
    """处理搜索结果，返回引用信息和文件名"""
    file_name = ""
    referrence = search_result.get("referrence", "")
    if referrence and "/books/" in referrence:
        file_name = referrence.split("/books/")[-1]
        file_name = os.path.splitext(file_name)[0]

    reference = {
        "index": str(reference_index),
        "id": search_result["id"],
        "content": search_result["content"],
        "file_path": search_result.get("file_path", ""),
        "title": search_result.get("title", ""),
        "distance": search_result.get("distance", DISTANCE_THRESHOLD),
        "page_no": search_result.get("page_no", ""),
        "file_name": file_name,
        "referrence": referrence
    }
    return reference, file_name


def _process_sentence_search(node_name: str, prop_title: str, sentences: list, trunks_service: TrunksService) -> tuple[
    list, list]:
    keywords = [node_name, prop_title] if node_name and prop_title else None
    return _process_sentence_search_keywords(sentences, trunks_service, keywords=keywords)


def _process_sentence_search_keywords(sentences: list, trunks_service: TrunksService,
                                      keywords: Optional[List[str]] = None) -> tuple[list, list]:
    """处理句子搜索，返回结果句子和引用列表"""
    result_sentences = []
    all_references = []
    reference_index = 1
    i = 0

    while i < len(sentences):
        sentence = sentences[i]
        search_text = sentence
        if keywords:
            search_text = f"{keywords}:{sentence}"

        i += 1

        # 使用向量搜索获取相似内容
        search_results = trunks_service.search_by_vector(
            text=search_text,
            limit=500,
            type='trunk',
            distance=0.7
        )

        # 准备语料库数据
        trunk_texts = []
        trunk_ids = []
        # 创建一个字典来存储trunk的详细信息
        trunk_details = {}

        for trunk in search_results:
            trunk_texts.append(trunk.get('content'))
            trunk_ids.append(trunk.get('id'))
            # 缓存trunk的详细信息
            trunk_details[trunk.get('id')] = {
                'id': trunk.get('id'),
                'content': trunk.get('content'),
                'file_path': trunk.get('file_path'),
                'title': trunk.get('title'),
                'referrence': trunk.get('referrence'),
                'page_no': trunk.get('page_no')
            }
        if len(trunk_texts) == 0:
            continue
        # 初始化TextSimilarityFinder并加载语料库
        similarity_finder = TextSimilarityFinder(method='tfidf', use_jieba=True)
        similarity_finder.load_corpus(trunk_texts, trunk_ids)

        # 使用TextSimilarityFinder进行相似度匹配
        similar_results = similarity_finder.find_most_similar(search_text, top_n=1)

        if not similar_results:  # 设置相似度阈值
            result_sentences.append({"sentence": sentence, "index": ""})
            continue

        # 获取最相似的文本对应的trunk_id
        trunk_id = similar_results[0]['path']

        # 从缓存中获取trunk详细信息
        trunk_info = trunk_details.get(trunk_id)

        if trunk_info:
            search_result = {
                **trunk_info,
                'distance': similar_results[0]['similarity']  # 转换相似度为距离
            }
            # 检查相似度是否达到阈值
            if search_result['distance'] >= DISTANCE_THRESHOLD:
                result_sentences.append({"sentence": sentence, "index": ""})
                continue
                # 检查是否已存在相同引用
            existing_ref = next((ref for ref in all_references if ref["id"] == search_result["id"]), None)
            current_index = int(existing_ref["index"]) if existing_ref else reference_index

            if not existing_ref:
                reference, _ = _process_search_result(search_result, reference_index)
                all_references.append(reference)
                reference_index += 1

            result_sentences.append({"sentence": sentence, "index": str(current_index)})

    return result_sentences, all_references
text_search_router = router