from contextlib import asynccontextmanager
from datetime import datetime
from typing import Optional

from fastapi import FastAPI, Depends, Security, HTTPException, Query
from py_tools.connections.http import AsyncHttpClient
from py_tools.logging import logger
from pydantic import BaseModel
from requests import Session

from .config.site import SiteConfig
from .db.session import get_db
from .middlewares.base import register_middlewares
from .model.response import StandardResponse
from .router.graph_api import graph_router
from .router.knowledge_nodes_api import knowledge_nodes_api_router, get_request_id, api_key_header
from .router.text_search import text_search_router
from .service.kg_edge_service import KGEdgeService
from .service.kg_node_service import KGNodeService
from .utils import log_util
from .utils.ObjectToJsonArrayConverter import ObjectToJsonArrayConverter


@asynccontextmanager
async def lifespan(app: FastAPI):
    await startup()
    yield
    await shutdown()


app = FastAPI(
    description="知识图谱开放平台",
    lifespan=lifespan,
    middleware=register_middlewares(),  # 注册web中间件
)
@app.get("/health")
async def health_check():
    """健康检查接口"""
    return {
        "status": "ok",
        "timestamp": datetime.utcnow().isoformat(),
        "service": "knowledge-graph"
    }


@app.post("/test", operation_id="医疗知识图谱目标节点查询", summary="根据医疗知识图谱获取医疗相关信息",
         description="""根据三元组的起始节点名称和关系名称，查询目标节点列表。
         该接口主要用于医疗知识图谱查询场景，例如：通过输入疾病名称和相关关系类型，
         返回该疾病对应的相关症状、治疗方法等信息。
         典型应用场景包括：
         - 症状查询：输入疾病名称和"疾病相关症状"关系
         - 诊断依据查询：输入疾病名称和"诊断依据"关系
         - 鉴别诊断查询：输入疾病名称和"疾病相关鉴别诊断"关系""",
         response_description="""返回目标节点名称的字符串列表，格式为：
         ["节点名称1", "节点名称2", ...]""")
async def test(node_name: str = Query(...,
                  description="""知识图谱三元组的起始节点名称，通常是疾病名称。
                 示例值：感冒、高血压、糖尿病等""",
                  example="糖尿病"),
                node_category: str = Query(...,
                  description="""知识图谱三元组的起始节点类型，通常是疾病。
                 示例值：疾病、症状等""",
                  example="疾病"),
                relation_name: str= Query(...,
                  description="""知识图谱三元组的关系名称，描述节点间的关系类型。
                 常见关系类型包括：
                 - 疾病相关症状
                 - 诊断依据
                 - 疾病相关鉴别诊断""",
                  example="疾病相关症状"), 
                db: Session = Depends(get_db)) -> list[str]:    
    """
    根据起始节点名称和关系名称查询目标节点名称列表
    
    参数:
        node_name: 起始节点名称(通常是疾病名称)
        relation_name: 关系类型名称
        
    返回:
        目标节点名称的字符串列表，如果查询不到结果则返回空列表
    """
    try:
        service = KGNodeService(db)
        search_params = {
            'keyword': node_name,
            'category': node_category,
            'pageNo': 1,
            'limit': 1,
            'load_props': False,
            'distance': 0.45,
        }
        node_list = service.paginated_search(search_params)
        edge_service = KGEdgeService(db)
        
        results = []
        if node_list and node_list.get('records'):
            first_node = node_list['records'][0]
            src_id = first_node['id']
            edges = edge_service.get_edges_by_nodes(src_id=src_id, dest_id=None,
                                                  name=relation_name)
           
            for edge in edges:
                dest_node = edge['dest_node']
                results.append(dest_node['name'])
        return results
    except Exception as e:
        logger.exception(f"分页查询失败: {str(e)}")
        raise HTTPException(
            status_code=500,
            detail=StandardResponse(
                success=False,
                error_code=500,
                error_msg=str(e)
            )
        )


async def init_setup():
    """初始化项目配置"""

    log_util.setup_logger()
    
async def startup():
    """项目启动时准备环境"""

    await init_setup()

    # 加载路由
    app.include_router(knowledge_nodes_api_router)
    app.include_router(text_search_router)
    app.include_router(graph_router)
    
    logger.info("fastapi startup success")


async def shutdown():
    await AsyncHttpClient.close()
    logger.error("app shutdown")
