"""
MCP客户端实现，使用fastmcp库连接SSE协议的MCP服务端

功能包括：
1. 建立与MCP服务端的连接
2. 发送消息到服务端
3. 接收服务端的SSE事件流
4. 错误处理和重连机制
"""
import fastmcp
import asyncio
from typing import Optional, Callable

class MCPClient:
    """
    MCP客户端类，用于连接SSE协议的MCP服务端
    
    参数:
        server_url (str): MCP服务端的URL地址
        on_message (Callable): 接收到消息时的回调函数
        reconnect_interval (int): 重连间隔时间(秒)
    """
    def __init__(self, server_url: str, on_message: Callable, reconnect_interval: int = 5):
        self.server_url = server_url
        self.on_message = on_message
        self.reconnect_interval = reconnect_interval
        self.client: Optional[fastmcp.Client] = None
        self.is_connected = False

    async def connect(self):
        """
        连接到MCP服务端
        
        如果连接失败会自动重试，间隔时间为reconnect_interval
        """
        while True:
            try:
                # 创建fastmcp客户端实例
                self.client = fastmcp.Client(self.server_url)
                
                # 建立连接
                await self.client.connect()
                self.is_connected = True
                print(f"成功连接到MCP服务端: {self.server_url}")
                
                # 开始监听SSE事件流
                await self._listen_events()
                
            except Exception as e:
                print(f"连接MCP服务端失败: {e}")
                self.is_connected = False
                
                # 等待重连间隔后重试
                await asyncio.sleep(self.reconnect_interval)
                print(f"尝试重新连接...")
                continue

    async def _listen_events(self):
        """
        监听SSE事件流
        
        这是一个内部方法，用于持续接收服务端推送的事件
        """
        try:
            async for event in self.client.listen():
                # 调用回调函数处理接收到的消息
                self.on_message(event.data)
                
        except Exception as e:
            print(f"监听事件流时出错: {e}")
            self.is_connected = False
            
    async def send_message(self, message: str):
        """
        发送消息到MCP服务端
        
        参数:
            message (str): 要发送的消息内容
        """
        if not self.is_connected:
            raise ConnectionError("客户端未连接到MCP服务端")
            
        try:
            await self.client.send(message)
            print(f"消息已发送: {message}")
            
        except Exception as e:
            print(f"发送消息失败: {e}")
            self.is_connected = False
            raise

    async def close(self):
        """
        关闭MCP客户端连接
        """
        if self.client and self.is_connected:
            await self.client.close()
            self.is_connected = False
            print("MCP客户端连接已关闭")


async def example_usage():
    """
    示例用法: 展示如何使用MCPClient类
    """
    def on_message_callback(data):
        """消息接收回调函数示例"""
        print(f"收到服务端消息: {data}")
    
    # 创建客户端实例
    client = MCPClient(
        server_url="http://localhost:8081/mcp",
        on_message=on_message_callback
    )
    
    try:
        # 启动客户端
        await client.connect()
        
        # 发送测试消息
        await client.send_message("Hello MCP Server!")
        
        # 保持运行
        while True:
            await asyncio.sleep(1)
            
    except KeyboardInterrupt:
        # 捕获Ctrl+C关闭客户端
        await client.close()


if __name__ == "__main__":
    # 运行示例
    asyncio.run(example_usage())